-- Most of the comments to the code (and the "well written" part of it :-) are by Phanx, whom I thanks for the helps and patience in explaining. 

local ADDON = ...

-- set some default values
GMAUTOPSY = {
     ENABLED = true,
     CHANNEL = 1,
}

GMAUTOPSY_OUTNAM = {"SELF", "GROUP", "GUILD", "OFFICER"}


local prgname = "|cffffd200gmAutopsy|r"
local string_format = string.format

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:NewDataObject(ADDON, {
	type = "data source",
	icon = "Interface\\Addons\\"..ADDON.."\\icon.tga",
	text = "-",
})

local frame = CreateFrame("Frame")
frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")

local function UpdateLDB()

	if (GMAUTOPSY["ENABLED"] == false) then
		dataobj.icon = "Interface\\Addons\\"..ADDON.."\\icon-off.tga"
		dataobj.text = string_format("|cffff0000%s|r", "OFF")
		frame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		-- print(string_format("%s: monitor is |cffff0000%s|r",prgname,"OFF"))
	else 
		dataobj.icon = "Interface\\Addons\\"..ADDON.."\\icon.tga"
		dataobj.text = string_format("|cff00ff00%s|r", GMAUTOPSY_OUTNAM[GMAUTOPSY["CHANNEL"]])
		frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		-- print(string_format("%s: enabled on [|cff00ff00%s|r] channel ", prgname, GMAUTOPSY_OUTNAM[GMAUTOPSY["CHANNEL"]]))
	end
	
end

function dataobj.OnClick(self, button)  

	if (button == "LeftButton" and GMAUTOPSY["ENABLED"] == true) then
		
		if (GMAUTOPSY["CHANNEL"] == 4) then 					
			GMAUTOPSY["CHANNEL"] = 1
		else
			GMAUTOPSY["CHANNEL"] = GMAUTOPSY["CHANNEL"] + 1
		end			
		
	elseif ( button == "RightButton" ) then				
		
		if (GMAUTOPSY["ENABLED"] == false) then  
			GMAUTOPSY["ENABLED"] = true			
		else 
			GMAUTOPSY["ENABLED"] = false
			
		end
		
	end
	
	UpdateLDB()
	
end

function dataobj.OnTooltipShow(tooltip)
	tooltip:AddLine(ADDON)	
	tooltip:AddLine(" ")
	tooltip:AddDoubleLine("Left Click", "Toggle Channel")
	tooltip:AddDoubleLine("Right Click", "Switch On|Off")
end


frame:SetScript("OnEvent", function(self, event, ...)
	
	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then	
	
		UpdateLDB()

	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then 
	
		local _, clevent, _, _, sourceName, _, _, destGUID, destName, _, _, prefixParam1, prefixParam2, _, suffixParam1, suffixParam2 = ...	
		local playerGUID
		local autopsy_msg = nil
 		
		-- proceed with CLEU handling
		if not playerGUID then 
			playerGUID = UnitGUID("player") 
		end
		
		if destGUID == playerGUID or (destName and UnitClass(destName)) then

			sourceName = sourceName or "Someone"
			
			if clevent == "SWING_DAMAGE" then
			
				if prefixParam2 > 0 then 
					autopsy_msg = string_format("gmAutopsy: %s killed %s with %s Melee overkill %s", sourceName, destName, prefixParam1, prefixParam2)
				end
			
			elseif clevent == "SPELL_DAMAGE" or clevent == "SPELL_PERIODIC_DAMAGE" or clevent == "RANGE_DAMAGE" then
			
				if suffixParam2 > 0 then 
					autopsy_msg = string_format("gmAutopsy: %s killed %s with %s damage of %s overkill %s", sourceName, destName, suffixParam1, GetSpellLink(prefixParam1), suffixParam2)
				end
				
			end
			
			
			if autopsy_msg then 
			
				local autopsy_chn = GMAUTOPSY_OUTNAM[GMAUTOPSY["CHANNEL"]]
			
				-- find the right GROUP, thanks to Dridzt code in the posts: http://www.wowinterface.com/forums/showthread.php?t=48320
				if GMAUTOPSY["CHANNEL"] == 2 then				
					if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
					  autopsy_chn = "INSTANCE_CHAT"
					elseif IsInRaid() then
					  autopsy_chn = "RAID"
					elseif IsInGroup() then
					  autopsy_chn = "PARTY"
					else
					  autopsy_chn = "SELF"
					end
				end
				
				if 	GMAUTOPSY["CHANNEL"] == "1" or autopsy_chn == "SELF" then 
					print(autopsy_msg)	
				else 
					SendChatMessage(autopsy_msg, autopsy_chn)
				end 

			end
			
		end
	end
end)